@@ -134,6 +134,9 @@ int main(int argc, char *argv[])
 	struct sockaddr_in addr;
 	int fd;
 	bool auth_ok;
+	struct l_certchain *cert = NULL;
+	struct l_key *priv_key = NULL;
+	struct l_queue *ca_cert = NULL;
 
 	if (argc != 2 && argc != 3 && argc != 6) {
 		printf("Usage: %s <https-host-name> [<ca-cert-path> "
@@ -189,15 +192,28 @@ int main(int argc, char *argv[])
 	if (getenv("TLS_DEBUG"))
 		l_tls_set_debug(tls, https_tls_debug_cb, NULL, NULL);
 
-	auth_ok = (argc <= 2 || l_tls_set_cacert(tls, argv[2])) &&
+	if (argc >= 3)
+		ca_cert = l_pem_load_certificate_list(argv[2]);
+
+	if (argc >= 4)
+		cert = l_pem_load_certificate_chain(argv[3]);
+
+	if (argc >= 6)
+		priv_key = l_pem_load_private_key(argv[4], argv[5], NULL);
+
+	auth_ok = (argc <= 2 || l_tls_set_cacert(tls, ca_cert)) &&
 		(argc <= 5 ||
-		 l_tls_set_auth_data(tls, argv[3], argv[4], argv[5])) &&
+		 l_tls_set_auth_data(tls, cert, priv_key)) &&
 		l_tls_start(tls);
 
 	if (tls && auth_ok)
 		l_main_run();
-	else
+	else {
 		printf("TLS setup failed\n");
+		l_queue_destroy(ca_cert, (l_queue_destroy_func_t) l_cert_free);
+		l_certchain_free(cert);
+		l_key_free(priv_key);
+	}
 
 	l_io_destroy(io);
 	l_tls_free(tls);
